#!/bin/bash
set +e
set -x

date --utc --iso-8601=seconds | xargs echo "[timestamp] Start time :"

if [ -z "$1" ] || [ -z "$2" ] || [ -z "$3" ]
then
    echo "You have to pass a device serial, packagearchive link and phablet password"
    echo "$0 04bf2c50e294d599 http://host/path/to/archive.zip ubuntuci"
    exit -1
fi

exec_with_adb() {
    adb -s $ANDROID_SERIAL shell "sudo $@"
}

exec_with_adb_user() {
    adb -s $ANDROID_SERIAL shell "$@"
}

ANDROID_SERIAL=$1
packagearchive=$2
phablet_password=$3
zip_name=`basename $packagearchive`
mir_rc=0
reboot_wait=30
network_wait=60
max_retries=5
results_dir=./results
mir_dir=$results_dir/mir
crash_dir=$results_dir/crash
log_dir=$results_dir/log
benchmark_dir=$results_dir/benchmark_results
benchmark_results=/tmp/glmark2_fullscreen_default.results

HOME_DIR="/home/phablet"
setup_dir="$HOME_DIR/mir-testrunner"

### Device initiation ###
sleep $reboot_wait

# Setup network
echo "SETTING UP NETWORK"
adb shell "echo $phablet_password | sudo -S nmcli dev wifi connect ubuntu-qa-g-wpa-d password qalabwireless"
phablet-network --skip-setup -t 90s

echo "SETTING UP WRITABLE-IMAGE"
phablet-config writable-image --remotepassword $phablet_password
sleep $reboot_wait

echo "SETTING UP PASSWORDLESS SUDO"
adb shell "echo $phablet_password | sudo -S bash -c 'echo phablet ALL=\(ALL\) NOPASSWD: ALL > /etc/sudoers.d/phablet && chmod 600 /etc/sudoers.d/phablet'"

exec_with_adb "system-image-cli -i"
echo "I: Turning on the display"
exec_with_adb "powerd-cli display on bright &"

echo "I: stopping lightdm"
exec_with_adb "/sbin/initctl stop lightdm"
    
### Cleanup ###
# Cleanup host
rm $zip_name
rm -rf $results_dir
rm -rf package_archive || true
mkdir -p $results_dir
mkdir -p $benchmark_dir
mkdir -p $mir_dir
mkdir -p $crash_dir
mkdir -p $log_dir

# Cleanup target
exec_with_adb "rm -r $setup_dir $HOME_DIR/output"
exec_with_adb "rm /var/crash/*"

date --utc --iso-8601=seconds | xargs echo "[timestamp] Start Setup target :"

### Setup target ###
# Setup scripts
exec_with_adb_user "mkdir -p $setup_dir"
adb -s $ANDROID_SERIAL push mir_install_packages.sh "$setup_dir"
adb -s $ANDROID_SERIAL push mir-smoke-test-runner.sh "$setup_dir"

# Collect pre-setup info
adb -s $ANDROID_SERIAL pull /etc/media-info $log_dir

# Setup package archive
wget $packagearchive
unzip $zip_name -d package_archive
if [ -d package_archive/archive/output ]; then
    ARCHIVE_DIR=package_archive/archive/output
    chmod 755 $ARCHIVE_DIR
else
    echo "E: Cannot find local package archive" >&2
    exit 1
fi
adb -s $ANDROID_SERIAL push $ARCHIVE_DIR $HOME_DIR/archive

date --utc --iso-8601=seconds | xargs echo "[timestamp] Start PPAs and local Archive :"

# Setup the PPAs and local Archive
ppa_repos="ppa_sources"
rm $ppa_repos || true
touch $ppa_repos
for h in $hooks; do
    echo "Processing $h"
    if echo $h | grep D09add_ppa; then
        ppa=`echo $h | sed 's/D09add_ppa~\(.*\)~\(.*\)/ppa:\1\/\2/'`
	echo ${ppa} >> ${ppa_repos}
    fi
done

local_archive="deb $local_archive_source/$local_archive_name $local_archive_pocket/ "
if echo $local_archive | grep "http"; then
    local_archive=`sh -c "echo $local_archive | sed \"s/naartjie/10.98.3.7/\""`
    echo ${local_archive} >> ${ppa_repos}
fi
echo "Host side PPA repo list:"
cat $ppa_repos
adb -s $ANDROID_SERIAL push ${ppa_repos} /tmp/${ppa_repos}

date --utc --iso-8601=seconds | xargs echo "[timestamp] Start Add the test packages :"

# Add the test packages (these are the extra packages specified in the job cfg)
exec_with_adb "apt-get -y --force-yes --no-install-recommends install $test_packages"


exec_with_adb "$setup_dir/mir_install_packages.sh"

### Pre-run ###
# Collect pre-run logs
exec_with_adb "top -n 1 -b | head -20 > /tmp/top-pre.log"
adb -s $ANDROID_SERIAL pull /tmp/top-pre.log $log_dir
exec_with_adb "dpkg -l > /tmp/dpkg-l.log"
adb -s $ANDROID_SERIAL pull /tmp/dpkg-l.log $log_dir

date --utc --iso-8601=seconds | xargs echo "[timestamp] Start Run Tests :"

### Run Tests ###
# Setup to run the tests
rm .setup_complete || true
adb -s $ANDROID_SERIAL pull /.setup_complete
if [[ ! -a .setup_complete ]]; then
    echo "SETUP FAILED - CAN'T CONTINUE"
    mir_rc=1
else
    set -x 
    for suite in $test_suites; do
        echo running suite $suite
        date --utc --iso-8601=seconds | xargs echo "[timestamp] Start :" $suite
        if grep -q mir_demo_client_ <<< $suite  ; then
            echo "I: ignoring $suite - now covered by smoke tests"
        elif grep -q mir_privileged_tests <<< $suite ; then
            if ! phablet-test-run -x -s $ANDROID_SERIAL $extra -v sudo /usr/bin/$suite ; then
                mir_rc=-1
            fi
        elif grep -q umock <<< $suite  ; then
            if ! phablet-test-run -x -s $ANDROID_SERIAL $extra -v sudo umockdev-run -- /usr/bin/$suite ; then
                mir_rc=-1
            fi
        else
            if ! phablet-test-run -x -s $ANDROID_SERIAL $extra -v sudo /usr/bin/$suite ; then
                mir_rc=-1
            fi
        fi

        date --utc --iso-8601=seconds | xargs echo "[timestamp] End :" $suite
    done

    adb -s $ANDROID_SERIAL wait-for-device
fi

# "smoke test" running all demo clients
if ! phablet-test-run -x -s $ANDROID_SERIAL $extra -v $setup_dir/mir-smoke-test-runner.sh ; then
    mir_rc=-1
fi

### Post Run ###
# Collect post-run logs
exec_with_adb "top -n 1 -b | head -20 > /tmp/top-post.log"
adb -s $ANDROID_SERIAL pull /tmp/top-post.log $log_dir
exec_with_adb "dmesg > /tmp/dmesg.log"
adb -s $ANDROID_SERIAL pull /tmp/dmesg.log $log_dir

for suite in $test_suites; do
    adb pull /tmp/$suite-log $mir_dir || true
done

# Collect crash files
echo "I: Gathering crash files (if any)"
adb -s $ANDROID_SERIAL pull /var/crash $crash_dir
ls $crash_dir

# Collect benchmark results
adb -s $ANDROID_SERIAL pull $benchmark_results $benchmark_dir

date --utc --iso-8601=seconds | xargs echo "[timestamp] End time :"

echo "I: Testing complete with returncode $mir_rc"
exit $mir_rc
